(*
 *    The MailUtl3Config, a configuration file generator for the MailUtl3.
 *    Copyright (C) 2006-2007  Fumio KAWAMATA
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the Licensel
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.
 *
 *)
unit MailUtl3Config_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls, Menus, FileCtrl, ImgList;

type
  TFormMain = class(TForm)
    GroupBoxMailUtlRules: TGroupBox;
    LabeledEditRuleFileName: TLabeledEdit;
    BitBtnSelectRuleFileName: TBitBtn;
    CheckBoxUseMailUtlRules: TCheckBox;
    GroupBoxS25R: TGroupBox;
    CheckBoxUseS25R: TCheckBox;
    GroupBoxNotification: TGroupBox;
    LabeledEditInsertionString: TLabeledEdit;
    MainMenu: TMainMenu;
    MainMenuFile: TMenuItem;
    MainMenuAbout: TMenuItem;
    MainMenuHelpAbout: TMenuItem;
    MainMenuSaveAs: TMenuItem;
    MainMenuFileExit: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    MainMenuOption: TMenuItem;
    MainMenuOptionFont: TMenuItem;
    MainMenuFileOpen: TMenuItem;
    CheckBoxAlsoNotifyInSubject: TCheckBox;
    ComboBoxDNSServer: TComboBox;
    LabelDNSServer: TLabel;
    ComboBoxMatchString: TComboBox;
    LabelMatchString: TLabel;
    FontDialog: TFontDialog;
    ImageList: TImageList;
    LabeledEditIniFileName: TLabeledEdit;
    LabelDummy: TLabel;
    LabeledEditMaxLines: TLabeledEdit;
    PanelButtons: TPanel;
    BitBtnOK: TBitBtn;
    BitBtnCancel: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure MainMenuHelpAboutClick(Sender: TObject);
    procedure MainMenuFileOpenClick(Sender: TObject);
    procedure LabeledEditRuleFileNameMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure MainMenuOptionFontClick(Sender: TObject);
    procedure MainMenuSaveAsClick(Sender: TObject);
    procedure MainMenuFileExitClick(Sender: TObject);
    procedure LabeledEditMaxLinesChange(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BitBtnSelectRuleFileNameClick(Sender: TObject);
  private
    FIniFileFullPath: string;
    procedure ReadParametersFromIniFile;
    procedure WriteParametersToIniFile;
    procedure GetCommandLineOptions;
  public
    property IniFileFullPath: string read FIniFileFullPath write FIniFileFullPath;
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

uses
  IniFiles, VerInfo, FontInfo, FormStatus;

var
  RuleFileName: string;

(* *****************************************************************************
 * GetCommandLineOptions
 * *****************************************************************************
 *)
procedure TFormMain.GetCommandLineOptions;
var
  I: Integer;
  S: String;
begin
  if ParamCount=0 then Exit;
  for I:=1 to ParamCount do
  begin
    S:=ParamStr(I);
    if Pos('-I', S)=1 then
    begin
      Delete(S, 1, 2);
      IniFileFullPath:=S;
    end
  end;
end;


(* *****************************************************************************
 * ReadParametersFromIniFile
 * *****************************************************************************
 *)
procedure TFormMain.ReadParametersFromIniFile;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(IniFileFullPath);
  LabeledEditIniFileName.Text:=IniFileFullPath;
  FormStatus.ReadFormStatusFromIniFile(FormMain, IniFileFullPath);
  FontInfo.ReadFontInfoFromIniFile(FormMain, IniFileFullPath);
  try
    CheckBoxUseMailUtlRules.Checked:=IniFile.ReadBool('MailUtlRules', 'UseMailUtlRules', False);
    LabeledEditRuleFileName.Text:=IniFile.ReadString('MailUtlRules', 'MailUtlRuleFileName', '');
    LabeledEditMaxLines.Text:=IntToStr(IniFile.ReadInteger('MailUtlRules', 'MaxLines', 20));

    CheckBoxUseS25R.Checked:=IniFile.ReadBool('S25R', 'UseS25R', False);
    ComboBoxDNSServer.Text:=IniFile.ReadString('S25R', 'DNSServer', '');
    ComboBoxMatchString.Text:=IniFile.ReadString('S25R', 'MatchString', '');

    CheckBoxAlsoNotifyInSubject.Checked:=IniFile.ReadBool('Notification', 'AlsoNotifyInSubject', False);
    LabeledEditInsertionString.Text:=IniFile.ReadString('Notification', 'InsertionString', '');
  except
    MessageDlg('Could not read parameters from '+IniFileFullPath+'.', mtWarning, [mbOK], 0);
  end;
  IniFile.Free;
end;




(* *****************************************************************************
 * WriteParametersToIniFile
 * *****************************************************************************
 *)
procedure TFormMain.WriteParametersToIniFile;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(IniFileFullPath);
  FormStatus.WriteFormStatusToIniFile(FormMain, IniFileFullPath);
  WriteFontInfoToIniFile(FormMain, IniFileFullPath);
  try
    IniFile.WriteBool('MailUtlRules', 'UseMailUtlRules', CheckBoxUseMailUtlRules.Checked);
    IniFile.WriteString('MailUtlRules', 'MailUtlRuleFileName', LabeledEditRuleFileName.Text);
    IniFile.WriteInteger('MailUtlRules', 'MaxLines', StrToIntDef(LabeledEditMaxLines.Text, 20));

    IniFile.WriteBool('S25R', 'UseS25R', CheckBoxUseS25R.Checked);
    IniFile.WriteString('S25R', 'DNSServer', ComboBoxDNSServer.Text);
    IniFile.WriteString('S25R', 'MatchString', ComboBoxMatchString.Text);

    IniFile.WriteBool('Notification', 'AlsoNotifyInSubject', CheckBoxAlsoNotifyInSubject.Checked);
    IniFile.WriteString('Notification', 'InsertionString', LabeledEditInsertionString.Text);
  except
    MessageDlg('Could not write parameters from '+IniFileFullPath+'.', mtWarning, [mbOK], 0);
  end;
  IniFile.Free;
  LabeledEditIniFileName.Text:=IniFileFullPath;
end;

(* *****************************************************************************
 * FormCreate
 * *****************************************************************************
 *)
procedure TFormMain.FormCreate(Sender: TObject);
begin
  LabelDummy.Left:=LabeledEditRuleFileName.Left;
  LabelDummy.Top:=LabeledEditRuleFileName.Top;
  LabelDummy.Width:=LabeledEditRuleFileName.Width;
  LabelDummy.Height:=LabeledEditRuleFileName.Height;

  VerInfo.GetVersionInfo;
  FormMain.Caption:=FormMain.Caption+'  '+VersionInfo.FileVersion;
  IniFileFullPath:=ChangeFileExt(ParamStr(0), '.ini');
  GetCommandLineOptions;
  ReadParametersFromIniFile;
  //ShowMessage('Form height='+IntToStr(FormMain.Height));
  //ShowMessage('Panel button top='+IntToStr(PanelButtons.Top));
  FormMain.Height:=PanelButtons.Top+PanelButtons.Height+60;
  //ShowMessage('Form height='+IntToStr(FormHeight));
end;



(* *****************************************************************************
 * FormDestroy
 * *****************************************************************************
 *)
procedure TFormMain.FormDestroy(Sender: TObject);
begin
  WriteParametersToIniFile;
end;


procedure TFormMain.MainMenuHelpAboutClick(Sender: TObject);
begin
  VerInfo.DisplayVersionInfo('');
end;

procedure TFormMain.MainMenuFileOpenClick(Sender: TObject);
//var
//  AdjustedFileName: string;
begin
  OpenDialog.DefaultExt:='.ini';
  OpenDialog.FileName:=IniFileFullPath;
  if OpenDialog.Execute=True then
  begin
    IniFileFullPath:=OpenDialog.FileName;
    ReadParametersFromIniFile;
    //RuleFileName:=OpenDialog.FileName;
    //AdjustedFileName:=MinimizeName(RuleFileName, LabelDummy.Canvas, LabeledEditRuleFileName.Width);
    //LabeledEditRuleFileName.Text:=AdjustedFileName;
  end;
end;


procedure TFormMain.BitBtnSelectRuleFileNameClick(Sender: TObject);
begin
  OpenDialog.DefaultExt:='.txt';
  OpenDialog.FileName:='*.txt';
  if OpenDialog.Execute=True then
  begin
    LabeledEditRuleFileName.Text:=OpenDialog.FileName;
  end;
end;




procedure TFormMain.LabeledEditRuleFileNameMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  LabeledEditRuleFileName.Text:=RuleFileName;
end;


(* *****************************************************************************
 * MainMenuOptionFontClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuOptionFontClick(Sender: TObject);
var
  I: Integer;
begin
  FontDialog.Font:=Font;
  if FontDialog.Execute=True then
  begin
    Font:= FontDialog.Font;
    for I:= 0 to Screen.FormCount-1 do
    begin
      Screen.Forms[I].Font:=Font;
    end;
  end;
end;

procedure TFormMain.MainMenuSaveAsClick(Sender: TObject);
begin
  SaveDialog.DefaultExt:='.ini';
  SaveDialog.Options:=[ofOverwritePrompt];
  SaveDialog.FileName:=IniFileFullPath;
  if SaveDialog.Execute=True then
  begin
    IniFileFullPath:=SaveDialog.FileName;
    WriteParametersToIniFile;
  end;
end;

procedure TFormMain.MainMenuFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.LabeledEditMaxLinesChange(Sender: TObject);
var
  V: Integer;
begin
  V:=StrToIntDef(LabeledEditMaxLines.Text, 20);
  if (IntToStr(V)<>LabeledEditMaxLines.Text)or(V<0) then
  begin
    MessageDlg('Invalid value.  The value will be set to 20', mtWarning, [mbOK], 0);
    LabeledEditMaxLines.Text:='20';
  end;
end;

procedure TFormMain.FormResize(Sender: TObject);
var
  X: Integer;
  W: Integer;
begin
  X:=FormMain.Width div 2;
  X:=X-(PanelButtons.Width div 2);
  if X<0 then
    X:=0;
  PanelButtons.Left:=X;

  W:=FormMain.Width-26;
  GroupBoxMailUtlRules.Width:=W;
  GroupBoxS25R.Width:=W;
  GroupBoxNotification.Width:=W;

  W:=FormMain.Width-LabeledEditRuleFileName.Left-40;
  LabeledEditRuleFileName.Width:=W;
  ComboBoxDNSServer.Width:=W;
  ComboBoxMatchString.Width:=W;
  LabeledEditInsertionString.Width:=W;
  LabeledEditIniFileName.Width:=W;
end;


end.


