(*
 *    The MailUtl3
 *    Copyright (C) 2002-2007  Fumio KAWAMATA
 *
 *    The author started adding the POP3 proxy function from December 2005 on.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License version 2 as
 *    published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License version 2 for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    version 2 along with this program.
 *
 *)
unit MailUtl3_Main;

interface

{$DEFINE Japanese}
(* Comment the compiler directive above if you do not use Japanese. *)


uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ShellAPI,
  IdServerIOHandler, IdServerIOHandlerSocket, IdObjs, IdStack, IdStackWindows,IdGlobal,
  IdServerIOHandlerStack, IdBaseComponent, IdComponent, IdPOP3Server, IdTCPClient,IdContext,
  StdCtrls, IdCustomTCPServer, Menus, ComCtrls, ImgList, ExtCtrls,
  IdCmdTCPServer, IdExplicitTLSClientServerBase, IdTCPServer, IdDNSResolver,
  IdTCPConnection, IdMessageClient, IdPOP3, IdYarn, IdCommandHandlers, IdException;
  (* Indy.Sockets Version 10.1.6 Copyright (c) 1993 - 2005,
   * Kudzu (Chad Z. Hower) and the Indy Pit Crew.
   *)


type
  TFormMain = class(TForm)
    Memo: TMemo;
    StatusBar: TStatusBar;
    MainMenu: TMainMenu;
    MainMenuFile: TMenuItem;
    MainMenuOption: TMenuItem;
    MainMenuOptionIniFileName: TMenuItem;
    MainMenuOptionFont: TMenuItem;
    FontDialog: TFontDialog;
    MainMenuOptionSeparator: TMenuItem;
    MainMenuFileSaveAs: TMenuItem;
    ImageListSystemTray: TImageList;
    MainMenuFileTerminate: TMenuItem;
    MainMenuFileHide: TMenuItem;
    MainMenuOptionSetting: TMenuItem;
    ImageList: TImageList;
    MainMenuHelp: TMenuItem;
    MainMenuHelpAbout: TMenuItem;
    TimerConnectionCheck: TTimer;
    IdPOP3Server: TIdPOP3Server;
    SaveDialog: TSaveDialog;
    MainMenuEdit: TMenuItem;
    MainMenuEditCopy: TMenuItem;
    MainMenuEditSelectAll: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure IdPOP3ServerConnect(AContext: TIdContext);
    procedure MainMenuOptionFontClick(Sender: TObject);
    procedure MainMenuFileExitClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MainMenuFileSaveAsClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MainMenuFileTerminateClick(Sender: TObject);
    procedure MainMenuFileHideClick(Sender: TObject);
    procedure MainMenuOptionSettingClick(Sender: TObject);
    procedure MainMenuHelpAboutClick(Sender: TObject);
    procedure TimerConnectionCheckTimer(Sender: TObject);
    procedure AddToMemo(Mess: string);
    procedure IdPOP3ServerCheckUser(aContext: TIdContext; aServerContext: TIdPOP3ServerContext);
    procedure IdPOP3ServerCAPA(aContext: TIdContext; aCapabilities: TStrings);
    procedure IdPOP3ServerSTAT(aCmd: TIdCommand; out oCount, oSize: Integer);
    procedure IdPOP3ServerLIST(aCmd: TIdCommand; AMsgNo: Integer);
    procedure IdPOP3ServerQUIT(aCmd: TIdCommand);
    procedure IdPOP3ServerUIDL(aCmd: TIdCommand; AMsgNo: Integer);
    procedure IdPOP3ServerDisconnect(AContext: TIdContext);
    procedure IdPOP3ServerRETRieve(aCmd: TIdCommand; AMsgNo: Integer);
    procedure CheckByS25R(DNSResolver: TIdDNSResolver; MyStack: TIdStackWindows; DNSServer: string; S25R_RuleList: TStringList; CheckIP: String; var HostName: String; var ReturnedIP: String; var Result: Integer);
    procedure HostNameToIPAddresses(DNSResolver: TIdDNSResolver; DNSServer: string; HostName: string; IPAddresses: TStringList);
    procedure IdPOP3ServerException(AContext: TIdContext; AException: Exception);
    procedure MainMenuEditCopyClick(Sender: TObject);
    procedure MainMenuEditSelectAllClick(Sender: TObject);
    procedure IdPOP3ServerTOP(aCmd: TIdCommand; aMsgNo, aLines: Integer);
    procedure IdPOP3ServerReSET(aCmd: TIdCommand);
    procedure IdPOP3ServerDELEte(aCmd: TIdCommand; AMsgNo: Integer);
    function ReadParametersFromIniFile(IniFileName: string): boolean;
    function WriteParametersToIniFile(IniFileName: string): boolean;
  private
    NID: TNotifyIconData;
    Icon: TIcon;
    IconIndex: Integer;
    S25R_RuleList: TStringList;
    FListeningPort: Integer; (* Default port number used to listen for new connections. *)
    FAllowConnectionFromOutside: Boolean;
    FMaxConnections: Integer;
    procedure DisplayIcon;
    procedure WMUser(var Msg: TMsg); message WM_USER+100;
  public
    property AllowConnectionFromOutSide: Boolean read FAllowConnectionFromOutside write FAllowConnectionFromOutside;
    property ListeningPort: Integer read FListeningPort write FListeningPort;
    property MaxConnections: Integer read FMaxConnections write FMaxConnections;
  end;


  TMyPOP3ServerContext = class(TIdPOP3ServerContext)
  private
    FAlsoNotifyInSubject: Boolean;
    FBody: TStringList;
    FCapabilities: TIdStrings;
    FDNSServer: string;
    FIdDNSResolver: TIdDNSResolver;
    FGreetings: string;
    FHeaderRaw: TStringList;
    FHeaderWork: TStringList;
    FHostName: string;
    FHostPort: Integer;
    FIdTCPClient: TIdTCPClient;
    FInsertionString: string;
    FMatchString: string;
    FMaxLines: Integer;
    FMyStack: TIdStackWindows;
    FUserName: string;
    procedure SetCapabilities(Value: TIdStrings);
  public
    constructor Create(AConnection: TIdTCPConnection; AYarn: TIdYarn;
      AList: TIdThreadList=nil); override;
    destructor Destroy; override;
    property AlsoNotifyInSubject: Boolean read FAlsoNotifyInSubject write FAlsoNotifyInSubject;
    property Body: TStringList read FBody write FBody;
    property Capabilities: TIdStrings read FCapabilities write SetCapabilities;
    property DNSResolver: TIdDNSResolver read FIdDNSResolver write FIdDNSResolver;
    property DNSServer: string read FDNSServer write FDNSServer;
    property Greetings: string read FGreetings write FGreetings;
    property HeaderRaw: TStringList read FHeaderRaw write FHeaderRaw;
    //property HeaderWork: TStringList read FHeaderWork write FHeaderWork;
    property HostName: string read FHostName write FHostName;
    property HostPort: Integer read FHostPort write FHostPort;
    property IdTCPClient: TIdTCPClient read FIdTCPClient;
    property InsertionString: string read FInsertionString write FInsertionString;
    property MatchString: string read FMatchString write FMatchString;
    property MaxLines: Integer read FMaxLines write FMaxLines;
    property MyStack: TIdStackWindows read FMyStack write FMyStack;
    property UserName: string read FUserName write FUserName;
  end;

  TDisplayInMemo = class(TThread)
  private
    MemoInThread: TMemo;
    MsgInThread: string;
    procedure DisplayIt;
  protected
    procedure Execute; override;
  published
    constructor Create( Memo: TMemo; MsgStr: string); virtual;
  end;


var
  FormMain: TFormMain;
  IniFileFullPath: String; (* Full path of the .INI file name. *)
//  RuleFileName: String;
//  UseMailUtlRules: Boolean;
//  UseS25R: Boolean;



implementation

{$R *.dfm}

uses
  MailUtl3_Settings,
  {$IFDEF Japanese}
  PzConv,
  (* PzConv.pas Copyright (C) 1998 Pz factory All rights Reserved.
   * PzConv.pas is a text converter.
   * PzConv.pas includes Japanese.
   *)
  {$ENDIF Japanese}
  BRegExp,
  (* BregExp.pas  Copyright (C) Osamu Takeuchi All rights Reserved.
   * BregExp.pas includes Japanese.
   * BregExp.pas requires BregExp.dll, Copyright (C) Tatsuo Baba.
   * BregExp.dll implements the regular expression.
   * BregExp.dll is free to use but its source code is not opened.
   * Check "Assignable typed constants" check box in "Compiler" tab,
   * "Project Options" dialog for using BregExp.pas.
   *)

  IniFiles, VerInfo, FontInfo, FormStatus, MailUtl3_Global, Splash;

const
  CRLF = chr($0D)+chr($0A);
  RegExpOfIPAddress='m/[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/';

var
  LogDir: String;
  {$IFDEF Japanese}
  Jap: TPzConv;
  {$ENDIF Japanese}
  //POP3Host: String;
  //POP3Port: Integer;
  //S25R_RuleList: TStringList;



procedure Logging(Msg: string);
var
  F: TextFile;
begin
  AssignFile(F, 'logfile.txt');
  Append(F);
  Writeln(F, Msg);
  CloseFile(F);
end;



(* *****************************************************************************
 * TMyPOP3ServerContext Class
 * *****************************************************************************
 *)
constructor TMyPOP3ServerContext.Create(AConnection: TIdTCPConnection;
  AYarn: TIdYarn; AList: TIdThreadList=nil);
begin
  inherited;
  FBody:=TStringList.Create;
  FCapabilities:=TIdStringList.Create;
  FHeaderRaw:=TStringList.Create;
  FHeaderWork:=TStringList.Create;
  FIdDNSResolver:=TIdDNSResolver.Create;
  FIdTCPClient:=TIdTCPClient.Create;
  FMyStack:=TIdStackWindows.Create;
end;

procedure TMyPOP3ServerContext.SetCapabilities(Value: TIdStrings);
begin
  FCapabilities.Assign(Value);
end;

destructor TMyPOP3ServerContext.Destroy;
begin

//sleep(5000);

{
    repeat
    //Application.ProcessMessages;
    until FIdTCPClient.IOHandler.Connected=False;
}
//  sleep(5000);
{
  FreeAndNil(FBody);
  FreeAndNil(FCapabilities);
  FreeAndNil(FHeaderRaw);
  FreeAndNil(FHeaderWork);
  FreeAndNil(FIdDNSResolver);
  FreeAndNil(FIdTCPClient);
}


  FBody.Free;
  FCapabilities.Free;
  FHeaderRaw.Free;
  FHeaderWork.Free;
  FIdDNSResolver.Free;
  FIdTCPClient.Free;
  FMyStack.Free;


//sleep(5000);

  //FHeaderWork.Free;



  //sleep(5000);

  inherited Destroy;
end;



(* *****************************************************************************
 * TDisplayInMemo Class
 * *****************************************************************************
 *)
constructor TDisplayInMemo.Create(Memo: TMemo; MsgStr: string);
begin
  MemoInThread:=Memo;
  MsgInThread:=MsgStr;
  inherited Create(False);
  FreeOnTerminate:=True;
end;

procedure TDisplayInMemo.DisplayIt;
begin
  If MemoInThread.Lines.Count>1000 then
    MemoInThread.Lines.Delete(0);
  MemoInThread.Lines.Add(MsgInThread);
end;

procedure TDisplayInMemo.Execute;
begin
  Synchronize(DisplayIt);
end;



(* *****************************************************************************
 * SetShortMonthNames
 *   ShortMonthNames[] is a built-in value.
 * *****************************************************************************
 *)
procedure SetShortMonthNames;
const
  ShortMonthNamesArray: array[1..12] of string = ('Jan', 'Feb', 'Mar', 'Apr',
    'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
var
  I: Integer;
begin
  for I:=1 to 12 do
    ShortMonthNames[I]:=ShortMonthNamesArray[I];
end;




(* *****************************************************************************
 * GetCommandLineOptions
 * *****************************************************************************
 *)
procedure GetCommandLineOptions;
var
  I: Integer;
  S: String;
begin
  if ParamCount=0 then Exit;
  for I:=1 to ParamCount do
  begin
    S:=ParamStr(I);
    if Pos('-I', S)=1 then
    begin
      Delete(S, 1, 2);
      MailUtl3_Main.IniFileFullPath:=S;
    end
  end;
end;





(* *****************************************************************************
 * SplitBeforeDelimiter
 * *****************************************************************************
 *)
function SplitBeforeDelimiter(var S: string; Delimiter: Char): string;
var
  DelimiterPos: Integer;
begin
  DelimiterPos:=Pos(Delimiter, S);
  if DelimiterPos=0 then
  begin
    Result:=Trim(S);
    S:='';
  end
  else
  if DelimiterPos=1 then
  begin
    Result:='';
    Delete(S, 1, 1);
  end
  else
  begin
    Result:=Trim(Copy(S, 1, DelimiterPos-1));
    Delete(S, 1, DelimiterPos);
  end;
end;



(* *****************************************************************************
 * ReadParametersFromIniFile
 * *****************************************************************************
 *)
function TFormMain.ReadParametersFromIniFile(IniFileName: string): boolean;
var
  IniFile: TIniFile;
begin
  ReadParametersFromIniFile:=True;
  IniFile := TIniFile.Create(IniFileName);
  try
    ListeningPort:=IniFile.ReadInteger('General', 'ListeningPort', 123);
    MaxConnections:=IniFile.ReadInteger('General', 'MaxConnections', 1);
    AllowConnectionFromOutside:=IniFile.ReadBool('General', 'AllowConnectionFromOutside', False);
    Logdir:=IniFile.ReadString('General', 'LogDirectory', 'C:\');
  except
    ReadParametersFromIniFile:=False;
  end;
  IniFile.Free;
end;



(* *****************************************************************************
 * WriteParametersToIniFile
 * *****************************************************************************
 *)
function TFormMain.WriteParametersToIniFile(IniFileName: string): boolean;
var
  IniFile: TIniFile;
begin
  WriteParametersToIniFile:=True;
  IniFile := TIniFile.Create(IniFileName);
  try
    IniFile.WriteInteger('General', 'ListeningPort', ListeningPort);
    IniFile.WriteInteger('General', 'MaxConnections', MaxConnections);
    IniFile.WriteBool('General', 'AllowConnectionFromOutside', AllowConnectionFromOutside);
    IniFile.WriteString('General', 'LogDirectory', LogDir);
  except
    WriteParametersToIniFile:=False;
  end;
  IniFile.Free;
end;




(* *****************************************************************************
 * LoadS25R_Rules
 * *****************************************************************************
 *)
function LoadS25R_Rules(S25R_RuleList: TStringList; RuleFileName: String): Boolean;
var
  F: text;
  S: String;
begin
  S25R_RuleList.Clear;
  Result:=True;
  try
    AssignFile(F, RuleFileName);
    Reset(F);
    while Eof(F)=False do
    begin
      Readln(F, S);
      S25R_RuleList.Add(S);
    end;
  except
    CloseFile(F);
    Result:=False;
  end;
end;


(* *****************************************************************************
 * MainMenuEditCopyClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuEditCopyClick(Sender: TObject);
begin
  Memo.CopyToClipboard;
end;

(* *****************************************************************************
 * MainMenuEditSelectAllClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuEditSelectAllClick(Sender: TObject);
begin
  Memo.SelectAll;
end;


(* *****************************************************************************
 * HostNameToIPAddresses
 * *****************************************************************************
 *)
procedure TFormMain.HostNameToIPAddresses(DNSResolver: TIdDNSResolver; DNSServer: string; HostName: string; IPAddresses: TStringList);
var
  Count: Integer;
  I: Integer;
begin
  IPAddresses.Clear;
  DNSResolver.Host:=DNSServer;
  DNSResolver.QueryType:=[qtA];
//AddToMemo('   P    MailUtl3 will execute Resolver.');
  try
    DNSResolver.Resolve(HostName);
    Count:=DNSResolver.QueryResult.Count;
  except
    Count:=0;
  end;

//AddToMemo('   P    MailUtl3 executed Resolver.');
  if Count=0 then exit;
  for I:=0 to Count-1 do
  begin
    if DNSResolver.QueryResult.Items[I] is TARecord then
      with TARecord(DNSResolver.QueryResult[I]) do
      begin
//AddToMemo('   P    IP address is '+IPAddress);
        IPAddresses.Add(IPAddress);
      end;
  end;
end;



(* *****************************************************************************
 * CheckByS25R
 * *****************************************************************************
 *)
procedure TFormMain.CheckByS25R(DNSResolver: TIdDNSResolver; MyStack: TIdStackWindows; DNSServer: string; S25R_RuleList: TStringList; CheckIP: String; var HostName: String; var ReturnedIP: String; var Result: Integer);
  (* Check S25R Rules
   *   Returns 1..n when the HostName matched the S25R rule 1..n
   *   Returns 0 when the HostName is not matched any S25R rule
   *   Returns -1 when the reverse lookup failure is occured
   *   Returns -2 when the forward lookup failure is occured
   *   Returns -3 when two IP addresses are different
   *)
var
  I: Integer;
  IPList: TStringList;
  Matched: Boolean;
  //LocalStack: TIdStackWindows;
begin
  (* Step1: Obtain HostName from CheckIP *)
//AddToMemo('   P    MailUtl3 will obtain hostname.');
  //LocalStack:=TIdStackWindows.Create;

  try
    //HostName:=GStack.HostByAddress(CheckIP, ID_DEFAULT_IP_VERSION);
    HostName:=MyStack.HostByAddress(CheckIP, ID_DEFAULT_IP_VERSION);
    //HostName:=LocalStack.HostByAddress(CheckIP, ID_DEFAULT_IP_VERSION);
  except
    HostName:='';
  end;
  if HostName='' then
  begin
    Result:=-1;
    exit;
  end;
  //HostName:='google.ne.jp';
  AddToMemo('   P    Sender''s hostname: '+HostName);
  //LocalStack.Free;

  (* Step2: Obtain forward lookuped IP addresses *)
  IPList:=TStringList.Create;

//AddToMemo('   P    MailUtl3 will obtain forward lookuped IP address. DNSServer='+DNSServer);
  HostNameToIPAddresses(DNSResolver, DNSServer, HostName, IPList);
//AddToMemo('   P    MailUtl3 obtained forward lookuped IP address.');
  if IPList.Count=0 then
  begin (* Number of IP addresses is zero *)
    IPList.Free;
    Result:=-2;
    exit;
  end
  else
  begin (* Number of IP addresses is greater than zero *)
//AddToMemo('   P    Number of IP addresses is '+IntToStr(IPList.Count)+'.');
    I:=0;
    ReturnedIP:='';
    Matched:=False;
    while (I<=IPList.Count-1)and(Matched=False) do
    begin
      if CheckIP=IPList[I] then
      begin
        Matched:=True;
        ReturnedIP:=CheckIP;
      end;
      inc(I);
    end;
    if Matched=False then
    begin
      ReturnedIP:=IPList[0]; (* Returns first IP address *)
      Result:=-3;
    end;
  end;
  IPList.Free;

  (* Step3: Check S25R Rules *)
//AddToMemo('   P    Step3');
  if S25R_RuleList.Count>0 then
  begin
    I:=0;
    Matched:=False;
    while (I<=S25R_RuleList.Count-1)and(Matched=False) do
    begin
      Matched:=brx.Match('m'+S25R_RuleList[I], HostName);
      inc(I);
    end;
    if Matched=True then
      Result:=I;
  end;
end;




(* *****************************************************************************
 * ArrangeHeader
 *   combine header lines which consist of several rows (ex: "Received:"
 *   header) into one line.
 * ****************************************************************************
 *)
procedure ArrangeHeader(var HeaderList: TStringList);
var
  Count:           Integer;
  FirstChar:       char;
  I:               Integer;
  LastField:       string;
  NumberOfHeaders: Integer;
  UpperStr:        string;
  TmpStr:          string;
begin
  NumberOfHeaders:=HeaderList.Count;
  LastField:='';
  I:=0; Count:=0;
  repeat
    TmpStr:=HeaderList[I];
    UpperStr:=UpperCase(TmpStr);
    FirstChar:=TmpStr[1];
    if (FirstChar=' ')or(FirstChar=char($09)) then
    begin
      if LastField='SUBJECT:' then
        HeaderList[I-1]:=HeaderList[I-1]+Trim(TmpStr)
      else
        HeaderList[I-1]:=HeaderList[I-1]+' '+Trim(TmpStr);
      HeaderList.Delete(I);
    end
    else
    begin
      HeaderList[I]:=TmpStr;
      if pos('SUBJECT:', UpperStr)=1 then
        LastField:='SUBJECT:'
      else
        LastField:='';
      inc(I);
    end;
    inc(Count);
  until Count>=NumberOfHeaders;
end;








(* *****************************************************************************
 * AddToMemo
 *
 *   This procedure adds the message into the Memo.
 * *****************************************************************************
 *)
procedure TFormMain.AddToMemo(Mess: string);
begin
  TDisplayInMemo.Create(Memo, Mess);
end;



(* *****************************************************************************
 * WMUser
 * User Window Message
 * *****************************************************************************
 *)
procedure TFormMain.WMUser(var Msg: TMsg);
begin
  if (Msg.wParam=WM_LBUTTONDOWN) or (Msg.wParam=WM_RBUTTONDOWN) then
  begin
    FormMain.Show;
    SetForegroundWindow(Application.Handle);
  end;
end;



(* *****************************************************************************
 * DisplayInIcon
 * Display Icon in the system tray
 * *****************************************************************************
 *)
procedure TFormMain.DisplayIcon;
begin
  ImageListSystemTray.GetIcon(IconIndex, Icon);
  NID.hIcon:=Icon.Handle;
  Shell_NotifyIcon(NIM_MODIFY, @NID);
end;



(* *****************************************************************************
 * FormCreate
 * *****************************************************************************
 *)
procedure TFormMain.FormCreate(Sender: TObject);
(*
var
  ExpireDate: TDateTime;
*)
begin
  inherited;
  IdPOP3Server.ContextClass:=TMyPOP3ServerContext;
  Self.BorderStyle:=bsSizeable;
  Self.BorderIcons:=[biSystemMenu,biMaximize]; (* Disable Minimize function. *)
  AllowConnectionFromOutside:=False;
  MailUtl3_Main.IniFileFullPath:=ChangeFileExt(ParamStr(0), '.ini');
  ListeningPort:=123;
  LogDir:='C:\';
  SetShortMonthNames;


  GetCommandLineOptions;
  VerInfo.GetVersionInfo;

  //POP3Host:='';
  //POP3Port:=110;

  ReadParametersFromIniFile(MailUtl3_Main.IniFileFullPath);
  FormStatus.ReadFormStatusFromIniFile(FormMain, MailUtl3_Main.IniFileFullPath);
  FontInfo.ReadFontInfoFromIniFile(FormMain, MailUtl3_Main.IniFileFullPath);
  FormMain.Caption:=FormMain.Caption+'  '+VersionInfo.FileVersion;
  FormMain.MainMenuOptionIniFileName.Caption:='INI file name: '+MailUtl3_Main.IniFileFullPath;
  {$IFDEF Japanese}
  Jap:=TPzConv.Create(Nil);
  {$ENDIF Japanese}

  S25R_RuleList:=TStringList.Create;
  LoadS25R_Rules(S25R_RuleList, 'S25R_Rules.txt');

  Memo.Lines.Add('Hello.  Welcome to the MailUtl3 Version '+VerInfo.VersionINfo.FileVersion);
  Memo.Lines.Add('MailUtl3.exe  Copyright (C) 2002-2007 Fumio Kawamata');
(*
  Memo.Lines.Add('The MailUtl3 is free software distributed under the GNU GPL verion 2.');
*)
  Memo.Lines.Add('');
  Memo.Lines.Add('The log will be displayed in this area.');
  Memo.Lines.Add('C->P     Command from client to server captured by proxy.');
  Memo.Lines.Add('   P->S  Command from client to server via proxy, or command from proxy to server.');
  Memo.Lines.Add('   P<-S  Response from server to client captured by proxy, or response from server to proxy.');
  Memo.Lines.Add('C<-P     Response from server to client via proxy, or response from proxy.');
  Memo.Lines.Add('   P     Internal message by proxy.');
  Memo.Lines.Add('The "proxy" means the MailUtl3, of course. :-)');
  Memo.Lines.Add('');

  (* Set parameters of the IdPOP3Server and activate it. *)
  IdPOP3Server.Greeting.Text.Clear;
  IdPOP3Server.Greeting.Text.Add('Welcome to the MailUtl3 Version '+VerInfo.VersionInfo.FileVersion);
  IdPOP3Server.DefaultPort:=ListeningPort;
  IdPOP3Server.MaxConnections:=MaxConnections;

  FormSplash.Free;

(*
  DateSeparator := '/';
  ShortDateFormat:='DD/MM/YY';
  ExpireDate:=StrToDate('16/09/07');
  if Date>=ExpireDate then
  begin
    MessageDlg('Expired.', mtError, [mbOK], 0);
    Application.Terminate;
  end;
*)
  try
    IdPOP3Server.Active:=True;
    Memo.Lines.Add('Port number: '+IntToStr(ListeningPort));
    Memo.Lines.Add('Started at '+FormatDateTime('mmm/dd/yyyy hh:nn:ss', Now));

    Icon:=TIcon.Create;
    ImageListSystemTray.GetIcon(0, Icon);
    NID.cbSize:=SizeOf(TNotifyIconData);
    NID.uID:=1;
    NID.Wnd:=Handle;
    NID.uFlags:=NIF_MESSAGE or NIF_ICON or NIF_TIP;
    NID.uCallbackMessage:=WM_USER+100;
    NID.hIcon:=Icon.Handle;
    NID.szTip:='MailUtl3';
    Shell_NotifyIcon(NIM_ADD, @NID);

  except
    on EIdCouldNotBindSocket do
    begin
      MessageDlg('The MailUtl3 could not bind socket.  The port ('+IntToStr(ListeningPort)+') is already in use.  The MailUtl3 will abort.', mtError, [mbOK], 0);
      Application.Terminate;
      (* The original error message is "Could not bind socket. Address and port are already in use.
       *)
    end;
  end;
end;



(* *****************************************************************************
 * FormDestroy
 * *****************************************************************************
 *)
procedure TFormMain.FormDestroy(Sender: TObject);
begin
  (* See TFormMain.MainMenuFileTerminateClick *)
end;



(* *****************************************************************************
 * Terminate
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuFileTerminateClick(Sender: TObject);
//var
  //I: Integer;
  //List: TIdList;
begin
  FormStatus.WriteFormStatusToIniFile(FormMain, MailUtl3_Main.IniFileFullPath);
  WriteFontInfoToIniFile(FormMain, MailUtl3_Main.IniFileFullPath);
  WriteParametersToIniFile(MailUtl3_Main.IniFileFullPath);
  {$IFDEF Japanese}
  Jap.Free;
  {$ENDIF Japanese}

  S25R_RuleList.Free;

  Shell_NotifyIcon(NIM_DELETE, @NID);
  Icon.Free;

  Application.Terminate;
  //Application.ProcessMessages;
end;



(* *****************************************************************************
 * IdPOP3ServerConnect
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerConnect(AContext: TIdContext);
var
  PeerIP: String;
  PeerPort: String;
  S: String;
begin
//  inherited;
//  IdPOP3Server.ContextClass:=TMyPOP3ServerContext;

  PeerIP:=AContext.Connection.Socket.Binding.PeerIP;
  PeerPort:=IntToStr(AContext.Connection.Socket.Binding.PeerPort);
  S:='New connection from '+PeerIP+':'+PeerPort+'.';
  S:='C->P    '+S;
  AddToMemo(S);


  (* Disconnect the connection if AllowConnectionFromOutside=False and
   * it is a connection from another host.
   *)

  if (AllowConnectionFromOutside=False)and(PeerIP<>'127.0.0.1') then
  begin

    S:='-ERR Only local connection is allowed.';
    AddToMemo('C<-P    '+S);
    AContext.Connection.IOHandler.WriteLn(S);
    AContext.Connection.Disconnect;
    Exit;
{
    S:='Only local connection is allowed.';
    AddToMemo('C<-P    -ERR '+S);
    raise Exception.Create(S);
    Exit;
}
{
    S:='-ERR Only local connection is allowed.';
    AddToMemo('C<-P    '+S);
    AContext.Connection.IOHandler.WriteLn(S);
    //AContext.Connection.Disconnect;
    Exit;
}
  end;

  S:='+OK '+IdPOP3Server.Greeting.Text[0];
  AddToMemo('C<-P    '+S);

  TimerConnectionCheck.Enabled:=True;
end;



(* *****************************************************************************
 * MainMenuOptionFontClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuOptionFontClick(Sender: TObject);
var
  I: Integer;
begin
  FontDialog.Font:=Font;
  if FontDialog.Execute=True then
  begin
    Font:= FontDialog.Font;
    for I:= 0 to Screen.FormCount-1 do
    begin
      Screen.Forms[I].Font:=Font;
    end;
  end;
end;



(* *****************************************************************************
 * MainMenuFileSaveAsClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuFileSaveAsClick(Sender: TObject);
begin
  SaveDialog.Title:='Save logged messages to a file';
  SaveDialog.Filter:='Text files (*.txt)|*.TXT|Log files (*.log)|*.LOG|All files (*.*)|*.*';
  SaveDialog.DefaultExt:='TXT';
  SaveDialog.Options:=[ofOverwritePrompt];
  SaveDialog.InitialDir:=LogDir;
  if SaveDialog.Execute=True then
  begin
    try
      Memo.Lines.SaveToFile(SaveDialog.FileName);
    except
      MessageDlg('Could not save "'+SaveDialog.FileName+'".', mtWarning, [mbOK], 0);
    end;
  end;
end;



(* *****************************************************************************
 * FormClose
 * *****************************************************************************
 *)
procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caNone;
  FormMain.Hide;
end;



(* *****************************************************************************
 * MainMenuFileHideClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuFileHideClick(Sender: TObject);
begin
  FormMain.Hide;
end;



(* *****************************************************************************
 * MainMenuFileExitClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuFileExitClick(Sender: TObject);
begin
  Application.Terminate;
end;



(* *****************************************************************************
 * MainMenuOptionSettingClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuOptionSettingClick(Sender: TObject);
begin
  if FormSettings.ShowModal=mrOK then
  begin
    (* Set properties *)
    ListeningPort:=StrToInt(FormSettings.LabeledEditListeningPort.Text);
    MaxConnections:=StrToInt(FormSettings.LabeledEditMaxConnections.Text);
    AllowConnectionFromOutside:=FormSettings.CheckBoxAllowConnectionFromOutside.Checked;
  end
  else begin
    (* Restore properties *)
    FormSettings.LabeledEditListeningPort.Text:=IntToStr(ListeningPort);
    FormSettings.LabeledEditMaxConnections.Text:=IntToStr(MaxConnections);
    FormSettings.CheckBoxAllowConnectionFromOutside.Checked:=AllowConnectionFromOutside;
  end;
end;



(* *****************************************************************************
 * MainMenuHelpAboutClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuHelpAboutClick(Sender: TObject);
var
  S: String;
begin
  S:='Portions of this software are Copyright (c) 1993 - 2004, Chad Z. Hower (Kudzu) and the Indy Pit Crew - http://www.nevrona.com/Indy/'+CRLF;
  S:=S+'PzConv.pas Copyright (C) 1998 Pz factory'+CRLF;
  S:=S+'BREGEXP.DLL Copyright (C) Tatsuo Baba'+CRLF;
  S:=S+'BregExp.pas Copyright (C) Osamu Takeuchi';
  VerInfo.DisplayVersionInfo(S);
end;



(* *****************************************************************************
 * TimerConnectionCheckTimer
 * *****************************************************************************
 *)
procedure TFormMain.TimerConnectionCheckTimer(Sender: TObject);
var
  NumberOfConnections: Integer;
begin
  TimerConnectionCheck.Enabled:=False;

  //NumberOfConnections:=IdPOP3Server.Contexts.Count;

  NumberOfConnections:=IdPOP3Server.Contexts.LockList.Count;
  IdPOP3Server.Contexts.UnlockList;

  if NumberOfConnections>0 then
  begin
    inc(IconIndex);
    if IconIndex>=ImageListSystemTray.Count then
      IconIndex:=1;
    DisplayIcon;
  end
  else begin
    IconIndex:=0;
    DisplayIcon;
  end;
  TimerConnectionCheck.Enabled:=True;
end;



(* *****************************************************************************
 * IdPOP3ServerDisconnect
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerDisconnect(AContext: TIdContext);
begin
  AddToMemo('   P    Disconnected');
end;



(* *****************************************************************************
 * IdPOP3ServerException
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerException(AContext: TIdContext;
  AException: Exception);
begin
  AddToMemo('   P    An exeption is occured.');
end;



(* *****************************************************************************
 * IdPOP3ServerCheckUser
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerCheckUser(aContext: TIdContext;
  aServerContext: TIdPOP3ServerContext);
var
  ConfigFileName:  string;
  ErrMsg:          string;
  IniFile:         TIniFile;
  ResponseOfPASS:  string;
  S:               string;
  ToActualSvr:     string;
  FromActualSvr:   string;
  ToMUA:           string;
begin
  (* aServerContext.Username holds the User name and
   * aServerContext.Password holds the Password.
   *)

  (* Format of the entry USER command
   * USER UserName#HostName:PortNumber;ConfigFileName
   *)

  S:=aServerContext.Username;
  (* The variable S holds UserName#HostName:PortNumber;ConfigFile
   *
   * Extract username for the actural POP3 server.
   * Raise error if the delimiter "#" is not found in the variable S.
   *)
  if Pos('#', S)=0 then
  begin
    ErrMsg:='Delimiter # is not found in the USER command.';
    AddToMemo('C<-P    -ERR '+ErrMsg);
    raise Exception.Create(ErrMsg);
    Exit;
  end;


  TMyPOP3ServerContext(aContext).UserName:=SplitBeforeDelimiter(S, '#');

  (* The variable S holds 'HostName:PortNumber;ConfigFileName'. *)

  (* Extract host name for the actural POP3 server.
   * Raise error if the delimiter ":" is not found in the variable S.
   *)
  if Pos(':', S)=0 then
  begin
    ErrMsg:='Delimiter : is not found in the USER command.';
    AddToMemo('C<-P     -ERR '+ErrMsg);
    raise Exception.Create(ErrMsg);
    Exit;
  end;
  TMyPOP3ServerContext(aContext).HostName:=SplitBeforeDelimiter(S, ':');
  (* The variable S holds "PortNumber;ConfigFileName". *)

  (* Extract port number for the actural POP3 server.
   * Raise error if the delimiter ";" is not found in the variable S.
   *)
  if Pos(';', S)=0 then
  begin
    ErrMsg:='Delimiter ; is not found in the USER command.';
    AddToMemo('C<-P     -ERR '+ErrMsg);
    raise Exception.Create(ErrMsg);
    Exit;
  end;
  TMyPOP3ServerContext(aContext).HostPort:=StrToIntDef(SplitBeforeDelimiter(S, ';'), 110);


  (* The variable S holds "ConfigFileName". *)
  ConfigFileName:=S;

  (* Raise error if the ConfigFileName is empty or not existed.
   *)
  if ConfigFileName='' then
  begin
    ErrMsg:='Config file name is empty.';
    AddToMemo('C<-P    -ERR '+ErrMsg);
    raise Exception.Create(ErrMsg);
    Exit;
  end;
  if not(FileExists(ConfigFileName)) then
  begin
    ErrMsg:='Config file "'+ConfigFileName+'" is not existed.';
    AddToMemo('C<-P    -ERR '+ErrMsg);
    raise Exception.Create(ErrMsg);
    Exit;
  end;

  (* Read parameters in the Configration file *)
  IniFile := TIniFile.Create(ConfigFileName);
  TMyPOP3ServerContext(aContext).MaxLines:=IniFile.ReadInteger('MailUtlRules', 'MaxLines', 20);
  TMyPOP3ServerContext(aContext).DNSServer:=IniFile.ReadString('S25R', 'DNSServer', '');
  TMyPOP3ServerContext(aContext).MatchString:=IniFile.ReadString('S25R', 'MatchString', '');
  TMyPOP3ServerContext(aContext).AlsoNotifyInSubject:=IniFile.ReadBool('Notification', 'AlsoNotifyInSubject', False);
  TMyPOP3ServerContext(aContext).InsertionString:=IniFile.ReadString('Notification', 'InsertionString', '');
  IniFile.Free;
  AddToMemo('   P    Max lines: '+IntToStr(TMyPOP3ServerContext(aContext).MaxLines));
  AddToMemo('   P    DNS Server: '+TMyPOP3ServerContext(aContext).DNSServer);
  AddToMemo('   P    Match string: '+TMyPOP3ServerContext(aContext).MatchString);

  (* Display received parameters in the memo *)
  AddToMemo('   P    User: '+TMyPOP3ServerContext(aContext).UserName
              +', Host: '+TMyPOP3ServerContext(aContext).HostName
              +', Port: '+IntToStr(TMyPOP3ServerContext(aContext).HostPort)
              +', ConfigFile: '+ConfigFileName);

  (* Connect to the actual POP3 server *)
  TMyPOP3ServerContext(aContext).IdTCPClient.Host:=TMyPOP3ServerContext(aContext).HostName;
  TMyPOP3ServerContext(aContext).IdTCPClient.Port:=TMyPOP3ServerContext(aContext).HostPort;
  AddToMemo('   P    Connecting to '+TMyPOP3ServerContext(aContext).HostName);
  TMyPOP3ServerContext(aContext).IdTCPClient.Connect;
  AddToMemo('   P    Connected');
  TMyPOP3ServerContext(aContext).Greetings:=TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+TMyPOP3ServerContext(aContext).Greetings);


  (* Send USER command to the actual POP3 server and receive its response *)
  ToActualSvr:='USER '+TMyPOP3ServerContext(aContext).UserName;
  AddToMemo('   P->S '+ToActualSvr);
  TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.WriteLn(ToActualSvr);
  FromActualSvr:=TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+FromActualSvr);
  if pos('-ERR', FromActualSvr)=1 then
  begin
    ToMUA:='MailUtl3 sent "'+ToActualSvr+'" but received "'+FromActualSvr+'" from the actual POP3 server.';
    AddToMemo('C<-P    '+ToMUA);
    raise Exception.Create(ToMUA);
    Exit;
  end;
  ResponseOfPASS:=FromActualSvr;

  (* Send PASS command to the actual POP3 server and receive its response *)
  ToActualSvr:='PASS '+aServerContext.Password;
  AddToMemo('   P->S '+'PASS ********');
  TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.Writeln(ToActualSvr);
  FromActualSvr:=TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+FromActualSvr);
  if pos('-ERR', FromActualSvr)=1 then
  begin
    ToMUA:='MailUtl3 sent "'+'USER '+TMyPOP3ServerContext(aContext).UserName+', PASS ********'+'" but received "'+FromActualSvr+'" from the actual POP3 server.';
    AddToMemo('C<-P    -ERR '+ToMUA);
    raise Exception.Create(ToMUA);
    Exit;
  end;


  (* Send CAPA command to the actual POP3 server and receive its response *)
  ToActualSvr:='CAPA';
  AddToMemo('   P->S '+ToActualSvr);
  TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.WriteLn(ToActualSvr);
  FromActualSvr:=TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+FromActualSvr);
  if pos('+OK', FromActualSvr)=1 then
  begin
    repeat
      FromActualSvr:=TMyPOP3ServerContext(aContext).IdTCPClient.IOHandler.ReadLn;
      AddToMemo('   P<-S '+FromActualSvr);
      if FromActualSvr<>'.' then
        TMyPOP3ServerContext(aContext).Capabilities.Add(FromActualSvr);
    until FromActualSvr='.';
  end;

end;



(* *****************************************************************************
 * IdPOP3ServerCAPA
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerCAPA(aContext: TIdContext;
  aCapabilities: TStrings);
var
  Caps: TIdStrings;
  I: Integer;
begin
  AddToMemo('C->P    CAPA');
  AddToMemo('C<-P    Response of CAPA command');
  Caps:=TMyPOP3ServerContext(aContext).Capabilities;
  if Caps.Count>0 then
  begin
    for I:=0 to Caps.Count-1 do
    begin
      if ACapabilities.IndexOf(Caps[I]) = -1 then
        ACapabilities.Add(Caps[I]);
    end;
  end;
end;



(* *****************************************************************************
 * IdPOP3ServerSTAT
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerSTAT(aCmd: TIdCommand; out oCount,
  oSize: Integer);
var
  Arg:           string;
  Response: string;
  S:             string;
  Command:   string;
begin
  AddToMemo('C->P    STAT');

  (* Send STAT command to the actual POP3 server and receive its response *)
  Command:='STAT';
  AddToMemo('   P->S STAT');
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.Writeln(Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+Response);
  if pos('-ERR', Response)=1 then
  begin
    AddToMemo('C<-P    '+Response);
    Delete(Response, 1, 5);
    raise Exception.Create(Response);
    Exit;
  end;

  S:=Response;
  Arg:=SplitBeforeDelimiter(S, ' '); (* Arg holds "+OK" *)
  Arg:=SplitBeforeDelimiter(S, ' '); (* Arg holds the number of e-mails *)
  oCount:=StrToIntDef(Arg, 0);
  Arg:=SplitBeforeDelimiter(S, ' '); (* Arg holds the total size of e-mails *)
  oSize:=StrToIntDef(Arg, 0);
  AddToMemo('C<-P    +OK '+IntToStr(oCount)+' '+IntToStr(oSize));
end;



(* *****************************************************************************
 * IdPOP3ServerLIST
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerLIST(aCmd: TIdCommand; AMsgNo: Integer);
var
  Response: string;
  Command: string;
begin
  (* AMsgNo is -1 if the message-number is not spedified *)
  Command:='LIST';
  if AMsgNo>-1 then
    Command:=Command+' '+IntToStr(AMsgNo);
  AddToMemo('C->P    '+Command);
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.Writeln(Command);
  AddToMemo('   P->S '+Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   C<-S '+Response);
  if pos('-ERR', Response)=1 then
  begin
    AddToMemo('C<-P    '+Response);
    Delete(Response, 1, 5);
    raise Exception.Create(Response);
    Exit;
  end;

  (* The response must be start "+OK" in this sate *)
  AddToMemo('C<-P    '+Response);
  Delete(Response, 1, 4);
  aCmd.Reply.SetReply(OK, Response);

  if AMsgNo=-1 then
  begin
    repeat
      Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
      if Response<>'.' then
      begin
        aCmd.Response.Add(Response);
      end;
    until Response='.';
  end;
end;



(* *****************************************************************************
 * IdPOP3ServerUIDL
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerUIDL(aCmd: TIdCommand; AMsgNo: Integer);
var
  Response: string;
  Command: string;
begin
  (* AMsgNo is -1 if the message-number is not spedified *)

  Command:='UIDL';
  if AMsgNo>-1 then
    Command:=Command+' '+IntToStr(AMsgNo);
  AddToMemo('C->P    '+Command);
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.Writeln(Command);
  AddToMemo('   P->S '+Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+Response);
  if pos('-ERR', Response)=1 then
  begin
    AddToMemo('C<-P    '+Response);
    Delete(Response, 1, 5);
    raise Exception.Create(Response);
    Exit;
  end;

  (* The response must be start "+OK" in this sate *)
  AddToMemo('C<-P    '+Response);
  Delete(Response, 1, 4);
  aCmd.Reply.SetReply(OK, Response);

  if AMsgNo=-1 then
  begin
    repeat
      Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
      if Response<>'.' then
      begin
        aCmd.Response.Add(Response);
      end;
    until Response='.';
  end;
end;



(* *****************************************************************************
 * IdPOP3ServerQUIT
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerQUIT(aCmd: TIdCommand);
var
  Response: string;
  Command: string;
begin
  AddToMemo('C->P    QUIT');
  Response:='Good-bye from the MailUtl3';
  aCmd.Reply.SetReply(OK, Response);
  AddToMemo('C<-P    +OK '+Response);
  if TMyPOP3ServerContext(aCmd.Context).IdTCPClient.Connected=True then
  begin
    Command:='QUIT';
    TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.Writeln('QUIT');
    AddToMemo('   P->S '+Command);
    Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
    AddToMemo('   P<-S '+Response);
  end;
end;




(* *****************************************************************************
 * IdPOP3ServerRETRieve
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerRETRieve(aCmd: TIdCommand;
  AMsgNo: Integer);
var
  //HeaderWork: TStringList;
  SenderIP: string;
  SenderHostName: string;
  I: Integer;
  IsBodyPart: Boolean;
  Matched: Boolean;
  Response: string;
  Result: Integer;
  LookupedIP: string;
  Command: string;
  S: string;
  X_Sender_IP_Address: string;
  X_Sender_Hostname: string;
  X_Lookuped_IP_Address: string;
  X_Status: string;
begin
  AddToMemo('C->P    RETR '+IntToStr(AMsgNo));

  (* Send TOP command and receive its response and stores
   * the response into the HeaderRaw and Body.
   *)
  Command:='TOP '+IntToStr(AMsgNo)+' '+IntToStr(TMyPOP3ServerContext(aCmd.Context).MaxLines);
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.Writeln(Command);
  AddToMemo('   P->S '+Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  if pos('-ERR', Response)=1 then
  begin
    AddToMemo('C<-P    '+Response);
    Delete(Response, 1, 5);
    raise Exception.Create(Response);
    Exit;
  end;
  (* The response must be start "+OK" in this state. *)
  TMyPOP3ServerContext(aCmd.Context).HeaderRaw.Clear;
  //TMyPOP3ServerContext(aCmd.Context).HeaderWork.Clear;
  TMyPOP3ServerContext(aCmd.Context).Body.Clear;
  repeat
    Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
    if Response<>'' then
    begin
      TMyPOP3ServerContext(aCmd.Context).HeaderRaw.Add(Response);
    end;
  until Response='';
  repeat
    Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
    if Response<>'.' then
    begin
      TMyPOP3ServerContext(aCmd.Context).Body.Add(Response);
    end;
  until Response='.';

  //TMyPOP3ServerContext(aCmd.Context).HeaderWork:=TMyPOP3ServerContext(aCmd.Context).HeaderRaw;
  //ArrangeHeader(TMyPOP3ServerContext(aCmd.Context).HeaderWork);
  //HeaderWork:=TStringList.Create;
  TMyPOP3ServerContext(aCmd.Context).FHeaderWork.Assign(TMyPOP3ServerContext(aCmd.Context).HeaderRaw);
  //HeaderWork.Assign(TMyPOP3ServerContext(aCmd.Context).HeaderRaw);
  ArrangeHeader(TMyPOP3ServerContext(aCmd.Context).FHeaderWork);

  (* Extract sender's IP address from arranged header.*)
  //I:=TMyPOP3ServerContext(aCmd.Context).HeaderWork.Count-1;
  I:=TMyPOP3ServerContext(aCmd.Context).FHeaderWork.Count-1;
  Matched:=False;
  SenderIP:='';
  while (I>=0)and(Matched=False) do
  begin
    //S:=TMyPOP3ServerContext(aCmd.Context).HeaderWork[I];
    S:=TMyPOP3ServerContext(aCmd.Context).FHeaderWork[I];
    if brx.Match('m/^Received:/i', S)=True then
    begin
      //if brx.Match(TMyPOP3ServerContext(aCmd.Context).MatchString, TMyPOP3ServerContext(aCmd.Context).HeaderWork[I])=True then
      if brx.Match('m'+TMyPOP3ServerContext(aCmd.Context).MatchString, TMyPOP3ServerContext(aCmd.Context).FHeaderWork[I])=True then
      begin
        Matched:=True;
      end;
    end;
    if Matched=False then
      dec(I);
  end;
  //HeaderWork.Free;

  if Matched=True then
  begin
    S:=brx[0];
    if brx.Match(RegExpOfIPAddress, S)=True then
    begin
      (* Sender's IP address is successfully extracted in this state. *)
      SenderIP:=brx[0];
      CheckByS25R(TMyPOP3ServerContext(aCmd.Context).DNSResolver,
        TMyPOP3ServerContext(aCmd.Context).MyStack,
        TMyPOP3ServerContext(aCmd.Context).DNSServer,
        S25R_RuleList, SenderIP,
        SenderHostName, LookupedIP, Result);
    end;
  end;
  if (Matched=True)and(SenderIP<>'') then
  begin
    X_Sender_IP_Address:=SenderIP;
    X_Sender_Hostname:=SenderHostName;
    X_Lookuped_IP_Address:=LookupedIP;
    if Result>0 then
      X_Status:='S25R Rule '+IntToStr(Result)
    else if Result=-1 then
      X_Status:='Reverse lookup failure'
    else if Result=-2 then
      X_Status:='Forward lookup failure'
    else if Result=-3 then
      X_Status:='Paranoid check failure'
    else
      X_Status:='Normal';
{
    case Result of
      0: X_Status:='Clean';
      1: X_Status:='S25R Rule 1';
      2: X_Status:='S25R Rule 2';
      3: X_Status:='S25R Rule 3';
      4: X_Status:='S25R Rule 4';
      5: X_Status:='S25R Rule 5';
     -1: X_Status:='Reverse lookup failure';
     -2: X_Status:='Forward lookup failure';
     -3: X_Status:='Paranoid check failure';
    end;
}
  end
  else begin
    X_Sender_IP_Address:='Unextracted';
    X_Sender_Hostname:='Unresolved';
    X_Lookuped_IP_Address:='Unresolved';
    X_Status:='Unknown';
  end;

  (* Send RETR command and receive its response. *)
  Command:='RETR '+IntToStr(AMsgNo);
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.Writeln(Command);
  AddToMemo('   P->S '+Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  if pos('-ERR', Response)=1 then
  begin
    AddToMemo('C<-P    '+Response);
    Delete(Response, 1, 5);
    raise Exception.Create(Response);
    Exit;
  end;

  (* The response must be start "+OK" in this sate *)
  Delete(Response, 1, 4);
  aCmd.Reply.SetReply(OK, Response);
  AddToMemo('C<-P    +OK '+Response);

  IsBodyPart:=False;
  repeat
    Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
    if Response<>'.' then
    begin
      if (IsBodyPart=False) and (Response='') then
      begin
        (* Insert the messages from the MailUtl3 here. *)
        aCmd.Response.Add('X-MailUtl3-Sender-IP-Address: '+X_Sender_IP_Address);
        aCmd.Response.Add('X-MailUtl3-Sender-Hostname: '+X_Sender_Hostname);
        aCmd.Response.Add('X-MailUtl3-Lookuped-IP-Address: '+X_Lookuped_IP_Address);
        aCmd.Response.Add('X-MailUtl3-Status: '+X_Status);
        IsBodyPart:=True;
      end;
      aCmd.Response.Add(Response);
    end;
  until Response='.';
end;



(* *****************************************************************************
 * IdPOP3ServerTOP
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerTop(aCmd: TIdCommand; aMsgNo,
  aLines: Integer);
var
  Command: string;
  Response: string;
begin
  Command:='TOP '+IntToStr(aMsgNo)+' '+IntToStr(aLines);
  (* Both message-number and non-negative number of lines are required values.
   * See RFC 1939.
   *)
  AddToMemo('C->P    '+Command);
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.WriteLn(Command);
  AddToMemo('   P->S '+Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+Response);
  if pos('-ERR', Response)=1 then
  begin
    AddToMemo('C<-P    '+Response);
    Delete(Response, 1, 5);
    raise Exception.Create(Response);
    Exit;
  end;

  (* The response must be start "+OK" in this stete *)
  AddToMemo('C<-P    '+Response);
  Delete(Response, 1, 4);
  aCmd.Reply.SetReply(OK, Response);
  repeat
    Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
    if Response<>'.' then
    begin
      aCmd.Response.Add(Response);
    end;
  until Response='.';
end;



(* *****************************************************************************
 * IdPOP3ServerReSET
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerReset(aCmd: TIdCommand);
var
  Command: string;
  Response: string;
begin
  Command:='RSET';
  AddToMemo('C->P    '+Command);
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.WriteLn(Command);
  AddToMemo('   P->S '+Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+Response);
  AddToMemo('C<-P    '+Response);
  Delete(Response, 1, 4);
  aCmd.Reply.SetReply(OK, Response);
end;



(* *****************************************************************************
 * IdPOP3ServerDELEte
 * *****************************************************************************
 *)
procedure TFormMain.IdPOP3ServerDelete(aCmd: TIdCommand; AMsgNo: Integer);
var
  Command: string;
  Response: string;
begin
  Command:='DELE '+IntToStr(aMsgNo);
  (* The message-number is required value.  See RFC 1939. *)
  AddToMemo('C->P    '+Command);
  TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.WriteLn(Command);
  AddToMemo('   P->S '+Command);
  Response:=TMyPOP3ServerContext(aCmd.Context).IdTCPClient.IOHandler.ReadLn;
  AddToMemo('   P<-S '+Response);
  if pos('-ERR', Response)=1 then
  begin
    AddToMemo('C<-P    '+Response);
    Delete(Response, 1, 5);
    raise Exception.Create(Response);
    Exit;
  end;
  (* The response must be start "+OK" in this stete *)
  AddToMemo('C<-P    '+Response);
  Delete(Response, 1, 4);
  aCmd.Reply.SetReply(OK, Response);
end;

end.
