(*
 *    The MailUtl3, a POP3 proxy.
 *    Copyright (C) 2002-2007  Fumio KAWAMATA
 *
 *    The POP3 proxy function is mounted on November 2005.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.
 *
 *)
unit MailUtl3_Settings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls;

type
  TFormSettings = class(TForm)
    BitBtnOK: TBitBtn;
    BitBtnCancel: TBitBtn;
    OpenDialog: TOpenDialog;
    GroupBoxConnection: TGroupBox;
    LabeledEditListeningPort: TLabeledEdit;
    LabeledEditMaxConnections: TLabeledEdit;
    CheckBoxAllowConnectionFromOutside: TCheckBox;
    GroupBoxLog: TGroupBox;
    LabeledEditLogDir: TLabeledEdit;
    BitBtnLogDir: TBitBtn;
    procedure BitBtnCancelClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BitBtnOKClick(Sender: TObject);
    procedure BitBtnRuleFileNameClick(Sender: TObject);
    procedure BitBtnLogDirClick(Sender: TObject);
    procedure LabeledEditListeningPortChange(Sender: TObject);
    procedure LabeledEditMaxConnectionsChange(Sender: TObject);
  private
    FIniFileFullPath: string;
  public
    property IniFileFullPath: string read FIniFileFullPath write FIniFileFullPath;
  end;

var
  FormSettings: TFormSettings;



implementation

{$R *.dfm}
uses
  FileCtrl, MailUtl3_Main, FormStatus;

var
  Msg: TMessage = (Msg: CM_PARENTFONTCHANGED);
  (* TMessage requires Messages unit. *)
  (* CM_PARENTFONTCHANGED requires Controls unit. *)


procedure TFormSettings.BitBtnCancelClick(Sender: TObject);
begin
  ModalResult:=mrCancel;
end;

procedure TFormSettings.FormDestroy(Sender: TObject);
begin
  //FormStatus.WriteFormStatusToIniFile(FormSettings, MailUtl3_Main.IniFileFullPath);
end;

procedure TFormSettings.FormCreate(Sender: TObject);
begin
  LabeledEditListeningPort.Text:=IntToStr(FormMain.ListeningPort);
  LabeledEditMaxConnections.Text:=IntToStr(FormMain.MaxConnections);
  CheckBoxAllowConnectionFromOutside.Checked:=FormMain.AllowConnectionFromOutSide;
end;



procedure TFormSettings.BitBtnOKClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;






procedure TFormSettings.BitBtnRuleFileNameClick(Sender: TObject);
begin
  OpenDialog.DefaultExt:='TXT';
  OpenDialog.InitialDir:=ExtractFileDir(ParamStr(0));
  OpenDialog.Filter := 'Text files (*.txt)|*.TXT|All files (*.*)|*.*';
  OpenDialog.Options:=[ofHideReadOnly, ofPathMustExist, ofFileMustExist, ofNoNetworkButton];
  if OpenDialog.Execute=True then
  begin
    //LabeledEditRuleFileName.Text:=OpenDialog.FileName;
  end;
end;


procedure TFormSettings.BitBtnLogDirClick(Sender: TObject);
var
  Dir: String;
begin
  Dir:= LabeledEditLogDir.Text;
  try
    if SelectDirectory(Dir, [sdAllowCreate, sdPerformCreate, sdPrompt], 0)=True then
    begin
      LabeledEditLogDir.Text:= Dir;
    end;
  except
    if MessageDlg('The directory in the textbox is not existed.  Do you reset it as a default directory?', mtConfirmation, [mbOK, mbNo], 0)=mrOK then
    begin
      LabeledEditLogDir.Text:=SysUtils.GetEnvironmentVariable('TEMP');
    end;
  end;
end;

procedure TFormSettings.LabeledEditListeningPortChange(Sender: TObject);
begin
  LabeledEditListeningPort.Text:=IntToStr(StrToIntDef(LabeledEditListeningPort.Text, 123));
end;

procedure TFormSettings.LabeledEditMaxConnectionsChange(Sender: TObject);
var
  V: Integer;
begin
  V:=StrToIntDef(LabeledEditMaxConnections.Text, 1);
  if V<1 then
    V:=1;
  LabeledEditMaxConnections.Text:=IntToStr(V);
end;

end.
