(* ****************************************************************************
 *                     Version Information Utility
 *                  Copyright (C) 1998-2006 Fumio Kawamata
 *
 *  You can get Version Information of the program by using this unit.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
 *  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.  You must be have it.
 * ****************************************************************************
 *)

unit VerInfo;

interface

type
  TVersionInfo = record
    CompanyName: String;
    FileDescription: String;
    FileVersion: String;
    InternalName: String;
    LegalCopyright: String;
    LegalTradeMarks: String;
    OriginalFileName: String;
    ProductName: String;
    ProductVersion: String;
    Comments: String;
  end;

var
  VersionInfo: TVersionInfo;

procedure DisplayVersionInfo(AdditionalMessage: String);
procedure GetVersionInfo;

implementation

uses
  Windows, Forms, Dialogs ;

procedure GetVersionInfo;
const
  Language     = '0411'; (* Japanese *)
  CharacterSet = '03A4'; (* Windows, Japanse(Shift JIS, X-0208) *)
var
  FullInfoSize: DWORD;
  ItemInfoSize: DWORD;
  pFullInfo: Pointer;
  pItemInfo: Pointer;
  Ret: DWORD;
begin
  FullInfoSize:=GetFileVersionInfoSize(pChar(Application.ExeName),Ret);
  if FullInfoSize>0 then begin
    GetMem(pFullInfo,FullInfoSize);
    try
      GetFileVersionInfo(PChar(Application.ExeName),0,FullInfoSize,pFullInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\CompanyName'),pItemInfo,ItemInfoSize);
      VersionInfo.CompanyName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\FileDescription'),pItemInfo,ItemInfoSize);
      VersionInfo.FileDescription:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\InternalName'),pItemInfo,ItemInfoSize);
      VersionInfo.InternalName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\FileVersion'),pItemInfo,ItemInfoSize);
      VersionInfo.FileVersion:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\LegalCopyright'),pItemInfo,ItemInfoSize);
      VersionInfo.LegalCopyright:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\LegalTradeMarks'),pItemInfo,ItemInfoSize);
      VersionInfo.LegalTradeMarks:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\OriginalFileName'),pItemInfo,ItemInfoSize);
      VersionInfo.OriginalFileName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\ProductName'),pItemInfo,ItemInfoSize);
      VersionInfo.ProductName:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\ProductVersion'),pItemInfo,ItemInfoSize);
      VersionInfo.ProductVersion:=PChar(pItemInfo);
      VerQueryValue(pFullInfo,PChar('\StringFileInfo\'+Language+CharacterSet+'\Comments'),pItemInfo,ItemInfoSize);
      VersionInfo.Comments:=PChar(pItemInfo);

    finally
      FreeMem(pFullInfo,FullInfoSize);
    end;
  end
end;

procedure DisplayVersionInfo;
const
  CRLF = chr($0D)+chr($0A);
var
  Mess : String;
begin
  GetVersionInfo;
  Mess:='Product name: '+VersionInfo.ProductName+CRLF
       +'Original file name: '+VersionInfo.OriginalFileName+CRLF
       +'File description: '+VersionInfo.FileDescription+CRLF
       +'File version: '+VersionInfo.FileVersion+CRLF
       +'Legal Copyright: '+VersionInfo.LegalCopyright+CRLF
       +'Comment: '+VerInfo.VersionInfo.Comments+CRLF
       +'Homepage: http://openlab.jp/fumio/';
  MessageDlg(Mess+CRLF+CRLF+AdditionalMessage,mtInformation,[mbOK],0);
end;

begin
  VersionInfo.CompanyName:='';
  VersionInfo.FileDescription:='';
  VersionInfo.FileVersion:='';
  VersionInfo.InternalName:='';
  VersionInfo.LegalCopyright:='';
  VersionInfo.OriginalFileName:='';
  VersionInfo.ProductName:='';
  VersionInfo.ProductVersion:='';
  VersionInfo.Comments:='';
end.
